/*
 * Jakelib2 - General purpose C++ library
 * Copyright (C) 2001 Florian Wolff  (florian@donuz.de)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: AbstractCollection.h,v 1.12 2003/02/18 17:01:19 alfa Exp $
 */

#ifndef INCLUDE_JAKELIB2_UTIL_ABSTRACTCOLLECTION_H
# define INCLUDE_JAKELIB2_UTIL_ABSTRACTCOLLECTION_H
#include "Collection.h"

namespace jakelib {
  namespace util {
    class Iterator;

    /**
     * A basic implementation of most of the methods in the Collection interface to
     * make it easier to create a collection. To create an unmodifiable Collection,
     * just subclass AbstractCollection and provide implementations of the
     * iterator() and size() methods. The Iterator returned by iterator() need only
     * provide implementations of hasNext() and next() (that is, it may throw an
     * UnsupportedOperationException if remove() is called). To create a modifiable
     * Collection, you must in addition provide an implementation of the
     * add(Object) method and the Iterator returned by iterator() must provide an
     * implementation of remove(). Other methods should be overridden if the
     * backing data structure allows for a more efficient implementation. The
     * precise implementation used by AbstractCollection is documented, so that
     * subclasses can tell which methods could be implemented more efficiently.
     *
     * The programmer should provide a no-argument constructor, and one that
     * accepts another Collection, as recommended by the Collection interface.
     * Unfortunately, there is no way to enforce this in Java.
     *
     * @class AbstractCollection
     * @version $Id: AbstractCollection.h,v 1.12 2003/02/18 17:01:19 alfa Exp $
     * @author Eric Blake <ebb9@email.byu.edu>
     * @author Florian 'Overflo' Wolff (florian@donuz.de)
     */
    class AbstractCollection : public jakelib::lang::Object, virtual public jakelib::util::Collection
      {
      protected:
	/**
	 * @brief The main constructor, for use by subclasses.
         */
	AbstractCollection();

      public:
	/**
	 * @brief Return an Iterator over this collection. The iterator must provide the
	 * hasNext and next methods and should in addition provide remove if the
	 * collection is modifiable.
	 *
	 * @return an iterator
         */
	virtual Iterator* iterator() = 0;

	/**
	 * Return the number of elements in this collection. If there are more than
	 * Integer.MAX_VALUE elements, return Integer.MAX_VALUE.
	 *
	 * @return the size
         */
	virtual int size() = 0;

	/**
	 * Add an object to the collection (optional operation). This implementation
	 * always throws an UnsupportedOperationException - it should be
	 * overridden if the collection is to be modifiable. If the collection
	 * does not accept duplicates, simply return false. Collections may specify
	 * limitations on what may be added.
	 *
	 * @param o the object to add
	 * @return true if the add operation caused the Collection to change
	 * @throws UnsupportedOperationException if the add operation is not
	 *         supported on this collection
	 * @throws NullPointerException if the collection does not support null
	 * @throws ClassCastException if the object is of the wrong type
	 * @throws IllegalArgumentException if some aspect of the object prevents
	 *         it from being added
         */
	virtual jboolean add(jakelib::lang::Object* o);

	/**
	 * Add all the elements of a given collection to this collection (optional
	 * operation). This implementation obtains an Iterator over the given
	 * collection and iterates over it, adding each element with the
	 * add(Object) method (thus this method will fail with an
	 * UnsupportedOperationException if the add method does). The behavior is
	 * unspecified if the specified collection is modified during the iteration,
	 * including the special case of trying addAll(this) on a non-empty
	 * collection.
	 *
	 * @param c the collection to add the elements of to this collection
	 * @return true if the add operation caused the Collection to change
	 * @throws UnsupportedOperationException if the add operation is not
	 *         supported on this collection
	 * @throws NullPointerException if this collection does not support null,
	 *         or if the specified collection is null
	 * @throws ClassCastException if an object in c is of the wrong type
	 * @throws IllegalArgumentException if some aspect of an object in c prevents
	 *         it from being added
	 * @see #add(Object)
         */
	virtual jboolean addAll(Collection* c);

	/**
	 * Remove all elements from the collection (optional operation). This
	 * implementation obtains an iterator over the collection and calls next
	 * and remove on it repeatedly (thus this method will fail with an
	 * UnsupportedOperationException if the Iterator's remove method does)
	 * until there are no more elements to remove.
	 * Many implementations will have a faster way of doing this.
	 *
	 * @throws UnsupportedOperationException if the Iterator returned by
	 *         iterator does not provide an implementation of remove
	 * @see Iterator#remove()
         */
	virtual void clear();

	/**
	 * Test whether this collection contains a given object. That is, if the
	 * collection has an element e such that (o == null ? e == null :
	 * o.equals(e)). This implementation obtains an iterator over the collection
	 * and iterates over it, testing each element for equality with the given
	 * object. If it is equal, true is returned. Otherwise false is returned when
	 * the end of the collection is reached.
	 *
	 * @param o the object to remove from this collection
	 * @return true if this collection contains an object equal to o
         */
	virtual jboolean contains(jakelib::lang::Object* o);

	/**
	 * Tests whether this collection contains all the elements in a given
	 * collection. This implementation iterates over the given collection,
	 * testing whether each element is contained in this collection. If any one
	 * is not, false is returned. Otherwise true is returned.
	 *
	 * @param c the collection to test against
	 * @return true if this collection contains all the elements in the given
	 *         collection
	 * @throws NullPointerException if the given collection is null
	 * @see #contains(Object)
         */
	virtual jboolean containsAll(Collection* c);

	/**
	 * Test whether this collection is empty. This implementation returns
	 * size() == 0.
	 *
	 * @return true if this collection is empty.
	 * @see #size()
         */
	virtual jboolean isEmpty();

	/**
	 * Remove a single instance of an object from this collection (optional
	 * operation). That is, remove one element e such that
	 * <code>(o == null ? e == null : o.equals(e))</code>, if such an element
	 * exists. This implementation obtains an iterator over the collection
	 * and iterates over it, testing each element for equality with the given
	 * object. If it is equal, it is removed by the iterator's remove method
	 * (thus this method will fail with an UnsupportedOperationException if
	 * the Iterator's remove method does). After the first element has been
	 * removed, true is returned; if the end of the collection is reached, false
	 * is returned.
	 *
	 * @param o The object to remove from this collection
	 * @return \c true if the remove operation caused the Collection to change, or
	 *         equivalently if the collection did contain o.
	 * @throw UnsupportedOperationException If this collection's Iterator
	 *         does not support the remove method
         */
	virtual jboolean remove(jakelib::lang::Object* o);

	/**
	 * Remove from this collection all its elements that are contained in a given
	 * collection (optional operation). This implementation iterates over this
	 * collection, and for each element tests if it is contained in the given
	 * collection. If so, it is removed by the Iterator's remove method (thus
	 * this method will fail with an UnsupportedOperationException if the
	 * Iterator's remove method does).
	 *
	 * @param c the collection to remove the elements of
	 * @return true if the remove operation caused the Collection to change
	 * @throw UnsupportedOperationException if this collection's Iterator
	 *         does not support the remove method
	 * @see Iterator#remove()
         */
	jboolean removeAll(Collection* c);

	/**
	 * Remove from this collection all its elements that are not contained in a
	 * given collection (optional operation). This implementation iterates over
	 * this collection, and for each element tests if it is contained in the
	 * given collection. If not, it is removed by the Iterator's remove method
	 * (thus this method will fail with an UnsupportedOperationException if
	 * the Iterator's remove method does).
	 *
	 * @param c the collection to retain the elements of
	 * @return true if the remove operation caused the Collection to change
	 * @throws UnsupportedOperationException if this collection's Iterator
	 *         does not support the remove method
	 * @see Iterator#remove()
         */
	virtual jboolean retainAll(Collection* c);

	/**
	 * Return an array containing the elements of this collection. This
	 * implementation creates an Object array of size size() and then iterates
	 * over the collection, setting each element of the array from the value
	 * returned by the iterator. The returned array is safe, and is not backed
	 * by the collection.
	 *
	 * @return an array containing the elements of this collection
         */
	//virtual jakelib::lang::Objects* toArray();

	/**
	 * Copy the collection into a given array if it will fit, or into a
	 * dynamically created array of the same run-time type as the given array if
	 * not. If there is space remaining in the array, the first element after the
	 * end of the collection is set to null (this is only useful if the
	 * collection is known to contain no null elements, however). This
	 * implementation first tests whether the given array is large enough to hold
	 * all the elements of the collection. If not, the reflection API is used to
	 * allocate a new array of the same run-time type. Next an iterator is
	 * obtained over the collection and the elements are placed in the array as
	 * they are returned by the iterator. Finally the first spare element, if
	 * any, of the array is set to null, and the created array is returned.
	 * The returned array is safe; it is not backed by the collection. Note that
	 * null may not mark the last element, if the collection allows null
	 * elements.
	 *
	 * @param a the array to copy into, or of the correct run-time type
	 * @return the array that was produced
	 * @throws NullPointerException if the given array is null
	 * @throws ArrayStoreException if the type of the array precludes holding
	 *         one of the elements of the Collection
         */
	//virtual jakelib::lang::Objects* toArray(jakelib::lang::Objects* a);

	/**
	 * Creates a String representation of the Collection. The string returned is
	 * of the form "[a, b, ...]" where a and b etc are the results of calling
	 * toString on the elements of the collection. This implementation obtains an
	 * Iterator over the Collection and adds each element to a StringBuffer as it
	 * is returned by the iterator.
	 *
	 * @return a String representation of the Collection
         */
	jakelib::lang::String* toString();

	/**
	 * Compare two objects according to Collection semantics.
	 *
	 * @param o1 the first object
	 * @param o2 the second object
	 * @return o1 == null ? o2 == null : o1.equals(o2)
         */
	static jboolean equals(jakelib::lang::Object* o1, jakelib::lang::Object* o2);

	/**
	 * Hash an object according to Collection semantics.
	 *
	 * @param o the object to hash
	 * @return o1 == null ? 0 : o1.hashCode()
         */	
	static int hashCode(jakelib::lang::Object* o);

      private:

	/**
	 * Remove from this collection all its elements that are contained in a given
	 * collection (optional operation). This implementation iterates over this
	 * collection, and for each element tests if it is contained in the given
	 * collection. If so, it is removed by the Iterator's remove method (thus
	 * this method will fail with an UnsupportedOperationException if the
	 * Iterator's remove method does). This method is necessary for ArrayList,
	 * which cannot publicly override removeAll but can optimize this call.
	 *
	 * @param c the collection to remove the elements of
	 * @return true if the remove operation caused the Collection to change
	 * @throws UnsupportedOperationException if this collection's Iterator
	 *         does not support the remove method
	 * @see Iterator#remove()
         */
	virtual jboolean removeAllInternal(Collection* c);

	/**
	 * Remove from this collection all its elements that are not contained in a
	 * given collection (optional operation). This implementation iterates over
	 * this collection, and for each element tests if it is contained in the
	 * given collection. If not, it is removed by the Iterator's remove method
	 * (thus this method will fail with an UnsupportedOperationException if
	 * the Iterator's remove method does). This method is necessary for
	 * ArrayList, which cannot publicly override retainAll but can optimize
	 * this call.
	 *
	 * @param c the collection to retain the elements of
	 * @return true if the remove operation caused the Collection to change
	 * @throws UnsupportedOperationException if this collection's Iterator
	 *         does not support the remove method
	 * @see Iterator#remove()
         */
	virtual jboolean retainAllInternal(Collection* c);

	JAKELIB_DECLARE_CLASS(AbstractCollection);
      };
  }
}
#endif

